@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 ##############################################################
 # smscon_service - remote control daemon for Nokia N900 phone#
-# $Id: smscon_service.py 139 2012-01-08 22:37:54Z yablacky $
+# $Id: smscon_service.py 141 2012-01-09 23:30:18Z yablacky $
 ##############################################################
 
 # standard modules
@@ -963,16 +963,6 @@ def PerformStatusReport():
 
     RevealThat('status', '\n'.join(StatusReport))
 
-def QueryDbus(DbusQueryCommand):
-    try:
-        Output = os.popen(DbusQueryCommand).read()
-        E = re.compile('string "(.*)"')
-        R = E.findall(Output)[0]
-    except:
-        LOGGER.error('Could not get telecom network operator name.')
-        return 'UNKNOWN'        
-
-
 def PerformReboot():
     """
     Reboot the phone.
