@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 ##############################################################
 # smscon - remote control library for Nokia N900 phone       #
-# $Id: smsconlib.py 124 2011-12-29 21:54:51Z yablacky $
+# $Id: smsconlib.py 145 2012-01-11 00:43:24Z yablacky $
 ##############################################################
 
 # standard modules
@@ -34,7 +34,7 @@ NAME = 'smscon'
 VERSION = '999.1'   # for trunk and private builds etc.
 
 NAME, VERSION = getSysBuildNameAndVersion(NAME, VERSION,
-    headerInfo  = '$Header: https://vcs.maemo.org/svn/smscon/branches/smscon_0.9.12/src/opt/smscon/smsconlib.py 124 2011-12-29 21:54:51Z yablacky $',
+    headerInfo  = '$Header: https://vcs.maemo.org/svn/smscon/branches/smscon_0.9.13/src/opt/smscon/smsconlib.py 145 2012-01-11 00:43:24Z yablacky $',
     dirLevels   = 3 )
     
 XDEBUG   = False        # cross debug (use only if not running on phone)
@@ -327,6 +327,9 @@ class Configuration(dict):
         LOGGER.info(Message)
         return ReturnCode
 
+    def _getPatternMismatchHint(self, ConfigVar, Pattern):
+        return getattr(Pattern, 'HelpText', 'Must match /%s/.' % Pattern)
+
     def _decode(self, ConfigVar, ConfigValue):
         return DecodeAES(AES.new(self._Secret), ConfigValue)
         
@@ -403,7 +406,8 @@ class Configuration(dict):
                                     self._upgradableConfigVars[ConfigVar] = True
                                     ErrorMsg = 'Value of %s "%s" needs upgrade.' % (self.ItemTypeName, ConfigVar)
                                 else:
-                                    self._warning('Value of %s "%s" does not match /%s/.' % (self.ItemTypeName, ConfigVar, Pattern))
+                                    self._warning('Value of %s "%s" is invalid. ' % (self.ItemTypeName, ConfigVar)
+                                                            + self._getPatternMismatchHint(ConfigVar, Pattern))
             if ErrorMsg:
                 if ok and not PrintContent: self._error('In %s:' % Thing)
                 print Line
@@ -454,7 +458,8 @@ class Configuration(dict):
                 
                 if Pattern and not re.match(Pattern, ConfigValue):
                     if not Silent:
-                        ok = self._warning('Value of %s "%s" does not match /%s/.' % (self.ItemTypeName, ConfigVar, Pattern))
+                        ok = self._warning('Value of %s "%s" is invalid. ' % (self.ItemTypeName, ConfigVar)
+                                                        + self._getPatternMismatchHint(ConfigVar, Pattern))
     
                 Line = self._genConfigLine(ConfigVar, self._genFileValue(ConfigValue, VarOptions))
                 f.writelines(Line + '\n')
@@ -549,7 +554,8 @@ class Configuration(dict):
 
             Pattern, DefaultValue, VarOptions, CommentLineIfNewOrOld = self._ConfigVars[ConfigVar]
             if Pattern and ConfigValue != None and not re.match(Pattern, ConfigValue):
-                return self._error('Value of %s "%s" does not match /%s/.' % (self.ItemTypeName, ConfigVar, Pattern))
+                return self._error('Value of %s "%s" is invalid. ' % (self.ItemTypeName, ConfigVar)
+                                                    + self._getPatternMismatchHint(ConfigVar, Pattern))
 
             if DefaultValue == None and ConfigValue != None:
                 return self._error('Obsolete %s "%s". %s' % (self.ItemTypeName, ConfigVar, CommentLineIfNewOrOld or ''))
@@ -707,17 +713,31 @@ class UserConfiguration(Configuration):
     """
     The SMScon user configuration database
     """
+    class pat(str):
+        """
+        Internal helper class for pattern to transport a user friendly help.
+        """
+        def help(self, HelpText):
+            self.HelpText = HelpText
+            return self
+
     ItemTypeName        = "user setting"
     CreateTemplate      = '# %s user settings (initially written by version %s)\n' % (NAME, VERSION)
     SmsCommandPreFix    = 'COM_'       # _ConfigVars with that prefix define commands that can be sent via SMS
     DefaultValueFormat  = '^\\+?[X.@]+$' # '+XXXXXXXXXXX', 'XXXX@XXXXXXXX.XX', 'XXXX.XXXXXXXX.XX', ...
-    _PhoneNumber        = '^\\+?[0-9]+$'
-    _CmdPattern         = ''
-    _BoolPattern        = '^(yes|no)$'
-    _IntPattern         = '^\\d+$'
-    _FloatPattern       = '^\\d+(\\.\\d+)?$'
-    _EmailPattern       = '^[^@]+@[^@]+$'
-    _IPforSSHPattern    = '^(\\*|localhost|\\d+(\\.\\d+)*)$'
+
+    __IPorHostPattern   = '(?:[A-Za-z0-9]+(?:[-.][A-Za-z0-9]+)*)'       # see RFC 952, RFC 1123.
+    _PhoneNumber        = pat('^(\\+{0,2}[0-9]+|)$')                    .help('Must be + and digits or empty.')
+    _CmdPattern         = pat('')
+    _BoolPattern        = pat('^(?:yes|no)$')                           .help('Must be "yes" or "no".')
+    _IntPattern         = pat('^\\d+$')                                 .help('Must be integer (no sign).')
+    _FloatPattern       = pat('^\\d+(?:\\.\\d+)?$')                     .help('Must be float-number (no sign).')
+    _HostnamePattern    = pat('^(?:'       + __IPorHostPattern + '|)$') .help('Must be IP addr or hostname or empty.')
+    _EmailPattern       = pat('^(?:[^@]+@' + __IPorHostPattern + '|)$') .help('Must be email address or empty.')
+    _URIPattern         = pat('^(?:([^:/?#]+:|)(?://|)(' + __IPorHostPattern + ')(:\\d+|)([/?#].*|)|)$'   # more restrictive than RFC 2396 but allow to be empty.
+                                                                    )   .help('Must be an URI (internet address) or empty.')
+    _IPforSSHPattern    = pat('^(?:\\*|localhost|\\d+(?:\\.\\d+)*)$')   .help('Must be "*" or "localhost" or integer.')
+
     _OPT_CRYPT          = Configuration._OPT_CRYPT
     _OPT_PLAIN          = Configuration._OPT_PLAIN
     _APPVERSIONVARNAME  = '@APPVERSION'
@@ -756,7 +776,7 @@ class UserConfiguration(Configuration):
 
         # REMOTE command source
         'ENABLECHECKHOST':  (_BoolPattern,      'no',               _OPT_PLAIN, '# Enable checking URL for commands:'),
-        'CHECKHOST':        ('.',        'http://example.com/file', _OPT_CRYPT, '# The URL to check (encoded):'),
+        'CHECKHOST':        (_URIPattern, 'http://example.com/file',_OPT_CRYPT, '# The URL to check (encoded):'),
         'CHECKTIME':        (_FloatPattern,     '15.0',             _OPT_PLAIN, '# Minutes between URL checks:'),
 
         # Email settings
@@ -764,11 +784,11 @@ class UserConfiguration(Configuration):
         'USER':             ('',                '',                 _OPT_CRYPT, '# SMTP username for EMAILFROM (encrypted):'),
         'PASSWORD':         ('',                '',                 _OPT_CRYPT, '# SMTP password for EMAILFROM (encrypted):'),
         'EMAILFROM':        (_EmailPattern,     'from@example.com', _OPT_CRYPT, '# Sender address of EMAIL notification (encrypted):'),
-        'MAILSERVER':       ('.',               'smtp.example.com', _OPT_CRYPT, '# SMTP server hostname or IP (encrypted):'),
+        'MAILSERVER':       (_HostnamePattern,  'smtp.example.com', _OPT_CRYPT, '# SMTP server hostname or IP (encrypted):'),
         'MAILPORT':         (_IntPattern,       '25',               _OPT_PLAIN, '# SMTP server port number:'),
 
         # SSH settings
-        'REMOTEHOST':       ('.',               'cmd.example.com',  _OPT_CRYPT, '# ssh server hostname or IP [to build tunnel] (encrypted):'),
+        'REMOTEHOST':       (_HostnamePattern,  'cmd.example.com',  _OPT_CRYPT, '# ssh server hostname or IP [to build tunnel] (encrypted):'),
         'REMOTEPORT':       (_IntPattern,       '22',               _OPT_PLAIN, '# ssh server port number [to build tunnel]:'),
         'REMOTEUSER':       ('',                'username',         _OPT_CRYPT, '# ssh server username [to build tunnel] (encrypted):'),
         'REMOTEPASSWORD':   ('',                'password',         _OPT_CRYPT, '# ssh server password [to build tunnel] (encrypted):'),
@@ -778,7 +798,7 @@ class UserConfiguration(Configuration):
 
         # Notification settings
         'COMMANDREPLY':     (_BoolPattern,      'no',               _OPT_PLAIN, '# Send notification if command accepted (yes|no):'),
-        'MESSAGESEND':      ('^sms|email|both|none$',   'sms',      _OPT_PLAIN, '# How to send notifications (sms|email|both|none):'),
+        'MESSAGESEND':      ('^(sms|email|both|none)$',     'sms',  _OPT_PLAIN, '# How to send notifications (sms|email|both|none):'),
         'ENABLERESEND':     (_BoolPattern,      'no',               _OPT_PLAIN, '# On failure try to re-send reply message (yes|no):'),
         'RESENDTIME':       (_FloatPattern,     '15.0',             _OPT_PLAIN, '# Minutes between message re-send tries:'),
         'RESENDMAXTRY':     (_IntPattern,       '10',               _OPT_PLAIN, '# Maximum number of message re-send tries (0=no limit):'),
@@ -789,7 +809,7 @@ class UserConfiguration(Configuration):
         'GPSTIMEOUT':       (_IntPattern,       '1800',             _OPT_PLAIN, '# Seconds until give up acquiring GPS coordinates:'),
         'GPSPOLLING':       (_IntPattern,       '3',                _OPT_PLAIN, '# Number of GPS coordinate acquisitions for COM_LOCATIONs best of:'),
         'GPSINTERVAL':      (_IntPattern,       '60',               _OPT_PLAIN, '# Seconds between GPS coordinate acquisitions for COM_TRACKON and COM_LOCATION (10..120:10|20|30|60|120)'),
-        'GPSSEND':          ('^sms|email|both$','sms',              _OPT_PLAIN, '# How to send GPS notifications (sms|email|both):'),
+        'GPSSEND':          ('^(sms|email|both)$',          'sms',  _OPT_PLAIN, '# How to send GPS notifications (sms|email|both):'),
 
         # Miscellaneous settings
         'SILENCEDEVICE':    (_BoolPattern,      'no',               _OPT_PLAIN, '# Set phone silent before executing command (yes|no):'),
